static const char *RcsId = "$Header: /home/cvsadm/cvsroot/fermi/servers/timer/src/Timer.cpp,v 1.1.1.1 2013-09-06 09:40:41 zambonl Exp $";
//+=============================================================================
//
// file :         Timer.cpp
//
// description :  C++ source for the Timer and its commands. 
//                The class is derived from Device. It represents the
//                CORBA servant object which will be accessed from the
//                network. All commands which can be executed on the
//                Timer are implemented in this file.
//
// project :      TANGO Device Server
//
// $Author: zambonl $
//
// $Revision: 1.1.1.1 $
//
// $Log: Timer.cpp,v $
// Revision 1.1.1.1  2013-09-06 09:40:41  zambonl
// Prima versione di timer
//
//
// copyleft :   Sincrotrone Trieste S.C.p.A.
//              Strada Statale 14 - km 163,5 in AREA Science Park
//              34012 Basovizza (TS)
//              ITALY
//
//-=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF 
//=============================================================================



//===================================================================
//
//	The following table gives the correspondance
//	between commands and method's name.
//
//  Command's name|  Method's name
//	----------------------------------------
//  State   |  dev_state()
//  Status  |  dev_status()
//  Stop    |  stop()
//  Start   |  start()
//
//===================================================================


#include <tango.h>
#include <Timer.h>
#include <TimerClass.h>

namespace Timer_ns
{

//+----------------------------------------------------------------------------
//
// method : 		Timer::Timer(string &s)
// 
// description : 	constructor for simulated Timer
//
// in : - cl : Pointer to the DeviceClass object
//      - s : Device name 
//
//-----------------------------------------------------------------------------
Timer::Timer(Tango::DeviceClass *cl,string &s)
:TANGO_BASE_CLASS(cl,s.c_str())
{
	init_device();
}

Timer::Timer(Tango::DeviceClass *cl,const char *s)
:TANGO_BASE_CLASS(cl,s)
{
	init_device();
}

Timer::Timer(Tango::DeviceClass *cl,const char *s,const char *d)
:TANGO_BASE_CLASS(cl,s,d)
{
	init_device();
}


/*
 * timestamp() 
 * send a timestamp to standard output
 */
void Timer::timestamp() 
{
	struct tm *mytime;
	time_t myTime_t;
	myTime_t = time(NULL);
	mytime = localtime(&myTime_t);
	strftime(timeLong, 79, "%Y-%m-%d %H:%M:%S ", mytime);
}

//+----------------------------------------------------------------------------
//
// method : 		supthread::msleep()
// 
// description : 	sleep for some milliseconds
//
//-----------------------------------------------------------------------------
void Timer::msleep(long milliseconds)
{
}

//+----------------------------------------------------------------------------
//
// method : 		Timer::delete_device()
// 
// description : 	will be called at device destruction or at init command.
//
//-----------------------------------------------------------------------------
void Timer::delete_device()
{
	//	Delete device's allocated object
}

//+----------------------------------------------------------------------------
//
// method : 		Timer::init_device()
// 
// description : 	will be called at device initialization.
//
//-----------------------------------------------------------------------------
void Timer::init_device()
{
	INFO_STREAM << "Timer::Timer() create device " << device_name << endl;

	// Initialise variables to default values
	//--------------------------------------------
	get_device_property();
	// alloc array used for status bits
	datab.length(3);
	startTime = 0;
}


//+----------------------------------------------------------------------------
//
// method : 		Timer::get_device_property()
// 
// description : 	Read the device properties from database.
//
//-----------------------------------------------------------------------------
void Timer::get_device_property()
{
	//	Initialize your default values here (if not done with  POGO).
	//------------------------------------------------------------------

	//	Read device properties from database.(Automatic code generation)
	//------------------------------------------------------------------
	Tango::DbData	dev_prop;
	dev_prop.push_back(Tango::DbDatum("SleepTime"));

	//	Call database and extract values
	//--------------------------------------------
	if (Tango::Util::instance()->_UseDb==true)
		get_db_device()->get_property(dev_prop);
	Tango::DbDatum	def_prop, cl_prop;
	TimerClass	*ds_class =
		(static_cast<TimerClass *>(get_device_class()));
	int	i = -1;


	//	End of Automatic code generation
	//------------------------------------------------------------------

}
//+----------------------------------------------------------------------------
//
// method : 		Timer::always_executed_hook()
// 
// description : 	method always executed before any command is executed
//
//-----------------------------------------------------------------------------
void Timer::always_executed_hook()
{
}

//+----------------------------------------------------------------------------
//
// method : 		Timer::read_attr_hardware
// 
// description : 	Hardware acquisition for attributes.
//
//-----------------------------------------------------------------------------
void Timer::read_attr_hardware(vector<long> &attr_list)
{
	DEBUG_STREAM << "Timer::read_attr_hardware(vector<long> &attr_list) entering... "<< endl;
	//	Add your own code here
}

//+----------------------------------------------------------------------------
//
// method : 		Timer::read_TimeToGo
// 
// description : 	Extract real attribute values for TimeToGo acquisition result.
//
//-----------------------------------------------------------------------------
void Timer::read_TimeToGo(Tango::Attribute &attr)
{
	DEBUG_STREAM << "Timer::read_TimeToGo(Tango::Attribute &attr) entering... "<< endl;
	long t;
	t = 0;
	if (startTime > 0) {
		t = requiredTime + startTime - time(NULL); 
	}
	attr_l_rd = t>0? t: 0;
	attr.set_value(&attr_l_rd);
}

//+----------------------------------------------------------------------------
//
// method : 		Timer::read_StateFlags
// 
// description : 	Extract real attribute values for StateFlags acquisition result.
//
//-----------------------------------------------------------------------------
void Timer::read_StateFlags(Tango::Attribute &attr)
{
	DEBUG_STREAM << "Timer::read_StateFlags(Tango::Attribute &attr) entering... "<< endl;
	datab[0] = (bool) (startTime>0);
	datab[1] = (bool) (startTime>0? time(NULL) - startTime > requiredTime: 0);
	datab[2] = !datab[1];
	attr_ab_rd = &datab[0];
	attr.set_value(attr_ab_rd, 3, 0, false);
}

//+------------------------------------------------------------------
/**
 *	method:	Timer::stop
 *
 *	description:	method to execute "Stop"
 *	stop and reset timer
 *
 *
 */
//+------------------------------------------------------------------
void Timer::stop()
{
	DEBUG_STREAM << "Timer::stop(): entering... !" << endl;

	//	Add your own code to control device here
	startTime = 0;
	timestamp(); cout << timeLong << " timer stopped" << endl;

}

//+------------------------------------------------------------------
/**
 *	method:	Timer::start
 *
 *	description:	method to execute "Start"
 *
 * @param	argin	time to sleep for
 *
 */
//+------------------------------------------------------------------
void Timer::start(Tango::DevDouble argin)
{
	DEBUG_STREAM << "Timer::start(): entering... !" << endl;

	//	Add your own code to control device here
	if (argin < 0) {
		startTime = 0;
		argin = -argin;
	}
	if (startTime == 0) {
		startTime = time(NULL);
		requiredTime = argin;
		timestamp(); cout << timeLong << " timer started for " << requiredTime << " seconds" << endl;
	}
	else {
		Tango::Except::throw_exception(
			(const char *) "timer is already running",
			(const char *) "Warning: already started",
			(const char *) "Timer::start() warning", Tango::WARN);
		return;
	}

}

}	//	namespace
